from flask import Flask, request, send_file, session, jsonify, Blueprint,abort
import os,uuid
from . import utils as ut
from wk import join_path
class Appication(Flask):
    pass
class NestableBlueprint(Blueprint):
    def __init__(
            self,
            name,
            import_name,
            static_folder=None,
            static_url_path=None,
            template_folder=None,
            url_prefix=None,
            subdomain=None,
            url_defaults=None,
            root_path=None,
            static_map=None,
            *args, **kwargs
    ):
        super().__init__(name=name, import_name=import_name, static_folder=static_folder,
                         static_url_path=static_url_path, template_folder=template_folder, url_prefix=url_prefix,
                         subdomain=subdomain, url_defaults=url_defaults, root_path=root_path,  *args, **kwargs)
        self.blueprints=[]
        self.static_map=static_map or {}
    def register(self, app, options, first_registration=False):
        # register self
        self.add_static_map_handlers()
        Blueprint.register(self,app,options,first_registration=first_registration)
        prefix=options.get('url_prefix',None)
        if prefix:
            self.url_prefix=prefix
        # register children blueprints
        for child in self.blueprints:
            bp=child['blueprint']
            register_options=child['register_options']
            bp_prefix=register_options.pop('url_prefix',bp.url_prefix)
            print(bp_prefix,self.url_prefix,register_options)
            if bp_prefix and self.url_prefix:
                bp_prefix=self.url_prefix+bp_prefix
                print(bp_prefix,self.url_prefix)
            app.register_blueprint(bp,url_prefix=bp_prefix,**register_options)
    def register_blueprint(self, blueprint, **options):
        self.blueprints.append(dict(
            blueprint=blueprint,register_options=options
        ))
    def add_statics(self,url_folder_map:dict):
        self.static_map.update(url_folder_map)
    def add_static(self, static_url_path, static_folder):
        self.static_map[static_url_path]=static_folder
    def add_static_map_handlers(self):
        if self.static_map:
            for url_prefix,folder in self.static_map.items():
                if not url_prefix.endswith('/'):url_prefix+='/'
                @self.route(url_prefix, defaults={'req_path': ''})
                @self.route(url_prefix + '<path:req_path>')
                @ut.rename_func("static-handler-" + uuid.uuid4().hex)
                def static_handler(req_path):
                    BASE_DIR = folder
                    abs_path = os.path.join(BASE_DIR, req_path)
                    abs_path = os.path.abspath(abs_path)
                    if not os.path.exists(abs_path):
                        return abort(404)
                    if os.path.isfile(abs_path):
                        return send_file(abs_path)
                    if os.path.isdir(abs_path):
                        fns = os.listdir(abs_path)
                        urls = [join_path(self.url_prefix, url_prefix, req_path, f) for f in fns]
                        dic=dict(zip(urls,fns))
                        string=[f'<li><a href="{url}">{filename}</a></li>' for url,filename in dic.items()]
                        string=f"<ul>{''.join(string)}</ul>"
                        return string





import functools,inspect
from flask import request

def rename_func(name):
    def decorator(func):
        func.__name__=name
        @functools.wraps(func)
        def new_func(*args,**kwargs):
            return func(*args,**kwargs)
        return new_func
    return decorator

def get_arg_dict(func):
    sign = inspect.signature(func)
    keys = list(sign.parameters.keys())
    dic = dict()
    for key in keys:
        value = sign.parameters.get(key).default
        dic[key] = value
    return dic


def parse_from(*refers):
    def decorator(f):
        arg_dict=get_arg_dict(f)
        fargs=list(arg_dict.keys())
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            dic = {}
            data_resource=[*refers,kwargs]
            for ref in data_resource:
                d = ref() if callable(ref) else dict(ref)
                d = d or {}
                if d: dic.update(d)
            params = {}
            for ag in fargs:
                val = dic.get(ag, None)
                if val is None:
                    for k, v in dic.items():
                        if k.replace('-', '_') == ag:
                            val = v
                if val is None:
                    val=arg_dict.get(ag,None)
                params[ag]=val
            params.update(kwargs)
            return f(*args, **params)

        return wrapper

    return decorator


def get_files(): return dict(request.files)


def get_form(): return request.form


def get_json(): return request.json


def get_cookies(): return request.cookies


def get_url_args(): return request.args


# parse_json is a decorator
parse_json_and_form = parse_from(get_json, get_form)
parse_json = parse_from(get_json)
parse_form = parse_from(get_form)
parse_files = parse_from(get_files)
parse_cookies = parse_from(get_cookies)
parse_args = parse_from(get_url_args)
parse_all = parse_from(get_cookies, get_form, get_json, get_url_args)


